
.set INT_DISABLED,           0xC0    /* Disable both FIQ and IRQ. */
.set MODE_IRQ,               0x12    /* IRQ Mode                  */
.set MODE_SVR,               0x13    /* Supervisor Mode           */

.extern currentSP
.extern scheduler

.global switch_to_current
.global SwitchingIRQHandler
.global SimpleIRQHandler

.code 32

/*
 *  Switch to current task without saving previous context.
 *  Used to run the first task the first time.
 *
 *  Pops the initial context setup up by the initialize_stack function.
 *  That means this function will act like a return to the task.
 *
 */
switch_to_current:
                                      /* Switch to the current task's stack. */
    ldr   r0, =currentSP              /* Load r0 with the address for currentSP */
    ldr   sp, [r0]                    /* Load sp with the value stored at the address */
                                      /* sp now points to the task's stack. */

                                      /* Set SPSR to the CPSR for the task. */
    ldmfd sp!, {r0}                   /* Pop the first value from the stack into r0. */
                                      /* That value was the CPSR for the task. */
    msr   spsr_cxsf, r0               /* Load SPSR with that CPSR value in r0. */

                                      /* Run task. */
    ldmfd sp!, {r0-r12, lr, pc}^      /* Pop the rest of the stack setting regs and pc for the task */
                                      /* Acts like a call the task. */


/*
 *  Simple IRQ handler that calls hello_world from the SVR mode.
 *
 *  It saves the context on the IRQ stack.
 *  Switches to the SVC mode and calls hello. Resets SVC LR after call.
 *  Switches back to IRQ mode and pops the context and returns.
 *
 *  Use this as an initial test.
 *
 */
SimpleIRQHandler:
    sub    lr, lr, #4                      /* LR offset to return from this exception: -4. */
    stmfd  sp!, {r0-r12, lr}               /* Push working registers. */

    /* Switch to SVR mode */

    msr    cpsr_c, #(INT_DISABLED | MODE_SVR)

    stmfd  sp!, {lr}                       /* Save LR before the call. */
    bl     hello_world                     /* Call hello_world. The branch w/ link overwrites LR. */
    ldmfd  sp!, {lr}                       /* Restore LR after the call. */

    /* Switch back to IRQ mode */

    msr    cpsr_c, #(INT_DISABLED | MODE_IRQ)

    ldmfd  sp!, {r0-r12, pc}^              /* Pop the context and returning to interrupted code. */


/*
 *  Context switching interrupt handler.
 *
 *  See "Instructions for implementing the context switch.txt" for full instructions.
 *
 */
SwitchingIRQHandler:
    /* IRQ mode with IRQ stack */

    /* Adjust LR back by 4 for use later. */

    /* Push working registers r0-r2 to the IRQ stack.
    /*  We'll use r0-r2 as variables. */

    /* Save task's cpsr (stored in the IRQ's spsr) to r0 */
    /* Save lr which is the task's pc to r1 */
    /* Save exception's stack pointer to r2 */

    /* Change to Supervisor mode with interrupts disabled. */

    /* SVR mode with SVR stack. This is the stack of interrupted task. */

    /* Push task's PC */
    /* Push task's LR */
    /* Push task's R3-R12 */

    /* Move task's r0-r3 from exception stack to task's stack. */
    /* By first loading r3-r5 with r0-r2 that where saved on the irq stack */
    /* then by pushing them onto the task's stack */

    /* Push task's CPSR, which was the IRQ mode's SPSR to the task stack. */

    /* Save task's stack pointer for call to switcher */
    /* Call the task scheduler which sets currentSP and resets the timer. */

    /* Set sp to the new task's sp by reading the value stored in currentSP. */

    /* Pop task's CPSR and restore it to SPSR. */
    /* SPSR will be moved to cpsr when we pop the context with LDMFD */

    /* Pop task's context. Restores regs and cpsr which reenables interrupts. */


